use indexmap::IndexMap;

use super::super::Spanned;

use super::super::Span;
use super::Lit;

#[derive(Debug, PartialEq)]
pub struct RawExpression<'src> {
    pub expr: Box<RawExpr<'src>>,
    pub span: Span,
}

impl<'src> RawExpression<'src> {
    pub fn new(expr: RawExpr<'src>, span: Span) -> Self {
        Self {
            expr: Box::new(expr),
            span,
        }
    }
}

#[derive(Debug, PartialEq)]
pub enum RawExpr<'src> {
    Node(
        Vec<Spanned<&'src str>>,
        Option<Spanned<IndexMap<Spanned<&'src str>, RawExpression<'src>>>>,
    ),
    SimplePipe(RawExpression<'src>, RawExpression<'src>),
    // NamingPipe(
    //     Box<Expression<'src>>,
    //     (Vec<Spanned<&'src str>>, Vec<Spanned<&'src str>>),
    //     Box<Expression<'src>>,
    // ),
    MappingPipe(RawExpression<'src>, RawExpression<'src>),
    NullPipe(RawExpression<'src>, RawExpression<'src>),
    MultiPipe(IndexMap<Spanned<&'src str>, RawExpression<'src>>),
    // LetIn(
    //     IndexMap<Spanned<&'src str>, Box<Expression<'src>>>,
    //     Box<Expression<'src>>,
    // ),
    // $
    Var(&'src str),
    // @
    InputVar(&'src str),
    AttrSet(Spanned<IndexMap<Spanned<&'src str>, RawExpression<'src>>>),
    Lit(Lit<'src>),
    Matrix(Spanned<(u16, u16)>, Vec<RawExpression<'src>>),
    List(Vec<RawExpression<'src>>),
}
